@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 2.7.  */
+/* A Bison parser, made by GNU Bison 2.5.1.  */
 
 /* Bison implementation for Yacc-like parsers in C
    
@@ -44,7 +44,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.7"
+#define YYBISON_VERSION "2.5.1"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -58,11 +58,14 @@
 /* Pull parsers.  */
 #define YYPULL 1
 
+/* Using locations.  */
+#define YYLSP_NEEDED 0
 
 
 
 /* Copy the first part of user declarations.  */
-/* Line 371 of yacc.c  */
+
+/* Line 268 of yacc.c  */
 #line 1 "parser.y"
 
 /* cfg_parser.y -- general-purpose configuration file parser
@@ -235,8 +238,9 @@ mu_cfg_create_node_list (mu_list_t *plist)
 }
 
 
-/* Line 371 of yacc.c  */
-#line 240 "parser.c"
+
+/* Line 268 of yacc.c  */
+#line 244 "parser.c"
 
 # ifndef YY_NULL
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -246,6 +250,11 @@ mu_cfg_create_node_list (mu_list_t *plist)
 #  endif
 # endif
 
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 1
+#endif
+
 /* Enabling verbose error messages.  */
 #ifdef YYERROR_VERBOSE
 # undef YYERROR_VERBOSE
@@ -254,18 +263,12 @@ mu_cfg_create_node_list (mu_list_t *plist)
 # define YYERROR_VERBOSE 0
 #endif
 
-/* In a future release of Bison, this section will be replaced
-   by #include "y.tab.h".  */
-#ifndef YY_YY_Y_TAB_H_INCLUDED
-# define YY_YY_Y_TAB_H_INCLUDED
-/* Enabling traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 1
-#endif
-#if YYDEBUG
-extern int mu_cfg_yydebug;
+/* Enabling the token table.  */
+#ifndef YYTOKEN_TABLE
+# define YYTOKEN_TABLE 0
 #endif
 
+
 /* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
@@ -286,10 +289,12 @@ extern int mu_cfg_yydebug;
 
 
 
+
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-/* Line 387 of yacc.c  */
+
+/* Line 295 of yacc.c  */
 #line 173 "parser.y"
 
   mu_cfg_node_t node;
@@ -301,36 +306,21 @@ typedef union YYSTYPE
   struct { const char *name; struct mu_locus locus; } ident;
 
 
-/* Line 387 of yacc.c  */
-#line 306 "parser.c"
+
+/* Line 295 of yacc.c  */
+#line 312 "parser.c"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define mu_cfg_yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
-extern YYSTYPE mu_cfg_yylval;
-
-#ifdef YYPARSE_PARAM
-#if defined __STDC__ || defined __cplusplus
-int mu_cfg_yyparse (void *YYPARSE_PARAM);
-#else
-int mu_cfg_yyparse ();
-#endif
-#else /* ! YYPARSE_PARAM */
-#if defined __STDC__ || defined __cplusplus
-int mu_cfg_yyparse (void);
-#else
-int mu_cfg_yyparse ();
-#endif
-#endif /* ! YYPARSE_PARAM */
-
-#endif /* !YY_YY_Y_TAB_H_INCLUDED  */
 
 /* Copy the second part of user declarations.  */
 
-/* Line 390 of yacc.c  */
-#line 334 "parser.c"
+
+/* Line 345 of yacc.c  */
+#line 324 "parser.c"
 
 #ifdef short
 # undef short
@@ -383,24 +373,24 @@ typedef short int mu_cfg_yytype_int16;
 # if defined YYENABLE_NLS && YYENABLE_NLS
 #  if ENABLE_NLS
 #   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
-#   define YY_(Msgid) dgettext ("bison-runtime", Msgid)
+#   define YY_(msgid) dgettext ("bison-runtime", msgid)
 #  endif
 # endif
 # ifndef YY_
-#  define YY_(Msgid) Msgid
+#  define YY_(msgid) msgid
 # endif
 #endif
 
 /* Suppress unused-variable warnings by "using" E.  */
 #if ! defined lint || defined __GNUC__
-# define YYUSE(E) ((void) (E))
+# define YYUSE(e) ((void) (e))
 #else
-# define YYUSE(E) /* empty */
+# define YYUSE(e) /* empty */
 #endif
 
 /* Identity function, used to suppress warnings about constant conditions.  */
 #ifndef lint
-# define YYID(N) (N)
+# define YYID(n) (n)
 #else
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
@@ -634,7 +624,7 @@ static const mu_cfg_yytype_uint16 mu_cfg_yyrline[] =
 };
 #endif
 
-#if YYDEBUG || YYERROR_VERBOSE || 0
+#if YYDEBUG || YYERROR_VERBOSE || YYTOKEN_TABLE
 /* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const mu_cfg_yytname[] =
@@ -722,10 +712,10 @@ static const mu_cfg_yytype_uint8 mu_cfg_yytable[] =
       12,    13,    25,    26,    28,    14,    32,    35,    38,     9
 };
 
-#define mu_cfg_yypact_value_is_default(Yystate) \
-  (!!((Yystate) == (-14)))
+#define mu_cfg_yypact_value_is_default(mu_cfg_yystate) \
+  ((mu_cfg_yystate) == (-14))
 
-#define mu_cfg_yytable_value_is_error(Yytable_value) \
+#define mu_cfg_yytable_value_is_error(mu_cfg_yytable_value) \
   YYID (0)
 
 static const mu_cfg_yytype_uint8 mu_cfg_yycheck[] =
@@ -789,18 +779,46 @@ do                                                              \
     }								\
 while (YYID (0))
 
-/* Error token number */
+
 #define YYTERROR	1
 #define YYERRCODE	256
 
 
+/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
+   If N is 0, then set CURRENT to the empty location which ends
+   the previous symbol: RHS[0] (always defined).  */
+
+#define YYRHSLOC(Rhs, K) ((Rhs)[K])
+#ifndef YYLLOC_DEFAULT
+# define YYLLOC_DEFAULT(Current, Rhs, N)				\
+    do									\
+      if (YYID (N))                                                    \
+	{								\
+	  (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;	\
+	  (Current).first_column = YYRHSLOC (Rhs, 1).first_column;	\
+	  (Current).last_line    = YYRHSLOC (Rhs, N).last_line;		\
+	  (Current).last_column  = YYRHSLOC (Rhs, N).last_column;	\
+	}								\
+      else								\
+	{								\
+	  (Current).first_line   = (Current).last_line   =		\
+	    YYRHSLOC (Rhs, 0).last_line;				\
+	  (Current).first_column = (Current).last_column =		\
+	    YYRHSLOC (Rhs, 0).last_column;				\
+	}								\
+    while (YYID (0))
+#endif
+
+
 /* This macro is provided for backward compatibility. */
+
 #ifndef YY_LOCATION_PRINT
 # define YY_LOCATION_PRINT(File, Loc) ((void) 0)
 #endif
 
 
 /* YYLEX -- calling `mu_cfg_yylex' with the right arguments.  */
+
 #ifdef YYLEX_PARAM
 # define YYLEX mu_cfg_yylex (YYLEX_PARAM)
 #else
@@ -863,7 +881,7 @@ mu_cfg_yy_symbol_value_print (mu_cfg_yyoutput, mu_cfg_yytype, mu_cfg_yyvaluep)
   switch (mu_cfg_yytype)
     {
       default:
-        break;
+	break;
     }
 }
 
@@ -1105,6 +1123,7 @@ mu_cfg_yysyntax_error (YYSIZE_T *mu_cfg_yymsg_alloc, char **mu_cfg_yymsg,
 {
   YYSIZE_T mu_cfg_yysize0 = mu_cfg_yytnamerr (YY_NULL, mu_cfg_yytname[mu_cfg_yytoken]);
   YYSIZE_T mu_cfg_yysize = mu_cfg_yysize0;
+  YYSIZE_T mu_cfg_yysize1;
   enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
   /* Internationalized format string. */
   const char *mu_cfg_yyformat = YY_NULL;
@@ -1167,13 +1186,11 @@ mu_cfg_yysyntax_error (YYSIZE_T *mu_cfg_yymsg_alloc, char **mu_cfg_yymsg,
                     break;
                   }
                 mu_cfg_yyarg[mu_cfg_yycount++] = mu_cfg_yytname[mu_cfg_yyx];
-                {
-                  YYSIZE_T mu_cfg_yysize1 = mu_cfg_yysize + mu_cfg_yytnamerr (YY_NULL, mu_cfg_yytname[mu_cfg_yyx]);
-                  if (! (mu_cfg_yysize <= mu_cfg_yysize1
-                         && mu_cfg_yysize1 <= YYSTACK_ALLOC_MAXIMUM))
-                    return 2;
-                  mu_cfg_yysize = mu_cfg_yysize1;
-                }
+                mu_cfg_yysize1 = mu_cfg_yysize + mu_cfg_yytnamerr (YY_NULL, mu_cfg_yytname[mu_cfg_yyx]);
+                if (! (mu_cfg_yysize <= mu_cfg_yysize1
+                       && mu_cfg_yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+                  return 2;
+                mu_cfg_yysize = mu_cfg_yysize1;
               }
         }
     }
@@ -1193,12 +1210,10 @@ mu_cfg_yysyntax_error (YYSIZE_T *mu_cfg_yymsg_alloc, char **mu_cfg_yymsg,
 # undef YYCASE_
     }
 
-  {
-    YYSIZE_T mu_cfg_yysize1 = mu_cfg_yysize + mu_cfg_yystrlen (mu_cfg_yyformat);
-    if (! (mu_cfg_yysize <= mu_cfg_yysize1 && mu_cfg_yysize1 <= YYSTACK_ALLOC_MAXIMUM))
-      return 2;
-    mu_cfg_yysize = mu_cfg_yysize1;
-  }
+  mu_cfg_yysize1 = mu_cfg_yysize + mu_cfg_yystrlen (mu_cfg_yyformat);
+  if (! (mu_cfg_yysize <= mu_cfg_yysize1 && mu_cfg_yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+    return 2;
+  mu_cfg_yysize = mu_cfg_yysize1;
 
   if (*mu_cfg_yymsg_alloc < mu_cfg_yysize)
     {
@@ -1258,27 +1273,32 @@ mu_cfg_yydestruct (mu_cfg_yymsg, mu_cfg_yytype, mu_cfg_yyvaluep)
     {
 
       default:
-        break;
+	break;
     }
 }
 
 
+/* Prevent warnings from -Wmissing-prototypes.  */
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int mu_cfg_yyparse (void *YYPARSE_PARAM);
+#else
+int mu_cfg_yyparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int mu_cfg_yyparse (void);
+#else
+int mu_cfg_yyparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
 
 
 /* The lookahead symbol.  */
 int mu_cfg_yychar;
 
-
-#ifndef YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
-# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
-# define YY_IGNORE_MAYBE_UNINITIALIZED_END
-#endif
-#ifndef YY_INITIAL_VALUE
-# define YY_INITIAL_VALUE(Value) /* Nothing. */
-#endif
-
 /* The semantic value of the lookahead symbol.  */
-YYSTYPE mu_cfg_yylval YY_INITIAL_VALUE(mu_cfg_yyval_default);
+YYSTYPE mu_cfg_yylval;
 
 /* Number of syntax errors so far.  */
 int mu_cfg_yynerrs;
@@ -1336,7 +1356,7 @@ mu_cfg_yyparse ()
   int mu_cfg_yyn;
   int mu_cfg_yyresult;
   /* Lookahead token as an internal (translated) token number.  */
-  int mu_cfg_yytoken = 0;
+  int mu_cfg_yytoken;
   /* The variables used to return semantic value and location from the
      action routines.  */
   YYSTYPE mu_cfg_yyval;
@@ -1354,8 +1374,9 @@ mu_cfg_yyparse ()
      Keep to zero when no symbol should be popped.  */
   int mu_cfg_yylen = 0;
 
-  mu_cfg_yyssp = mu_cfg_yyss = mu_cfg_yyssa;
-  mu_cfg_yyvsp = mu_cfg_yyvs = mu_cfg_yyvsa;
+  mu_cfg_yytoken = 0;
+  mu_cfg_yyss = mu_cfg_yyssa;
+  mu_cfg_yyvs = mu_cfg_yyvsa;
   mu_cfg_yystacksize = YYINITDEPTH;
 
   YYDPRINTF ((stderr, "Starting parse\n"));
@@ -1364,6 +1385,14 @@ mu_cfg_yyparse ()
   mu_cfg_yyerrstatus = 0;
   mu_cfg_yynerrs = 0;
   mu_cfg_yychar = YYEMPTY; /* Cause a token to be read.  */
+
+  /* Initialize stack pointers.
+     Waste one element of value and location stack
+     so that they stay on the same level as the state stack.
+     The wasted elements are never initialized.  */
+  mu_cfg_yyssp = mu_cfg_yyss;
+  mu_cfg_yyvsp = mu_cfg_yyvs;
+
   goto mu_cfg_yysetstate;
 
 /*------------------------------------------------------------.
@@ -1504,9 +1533,7 @@ mu_cfg_yybackup:
   mu_cfg_yychar = YYEMPTY;
 
   mu_cfg_yystate = mu_cfg_yyn;
-  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   *++mu_cfg_yyvsp = mu_cfg_yylval;
-  YY_IGNORE_MAYBE_UNINITIALIZED_END
 
   goto mu_cfg_yynewstate;
 
@@ -1543,7 +1570,8 @@ mu_cfg_yyreduce:
   switch (mu_cfg_yyn)
     {
         case 2:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 196 "parser.y"
     {
 	    parse_node_list = (mu_cfg_yyvsp[(1) - (1)].nodelist);
@@ -1551,7 +1579,8 @@ mu_cfg_yyreduce:
     break;
 
   case 3:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 202 "parser.y"
     {
 	    mu_cfg_create_node_list (&(mu_cfg_yyval.nodelist));
@@ -1560,7 +1589,8 @@ mu_cfg_yyreduce:
     break;
 
   case 4:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 207 "parser.y"
     {
 	    mu_list_append ((mu_cfg_yyvsp[(1) - (2)].nodelist), (mu_cfg_yyvsp[(2) - (2)].pnode));
@@ -1570,7 +1600,8 @@ mu_cfg_yyreduce:
     break;
 
   case 7:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 219 "parser.y"
     {
 	    (mu_cfg_yyval.pnode) = mu_cfg_alloc_node (mu_cfg_node_param, &(mu_cfg_yyvsp[(1) - (3)].ident).locus,
@@ -1580,7 +1611,8 @@ mu_cfg_yyreduce:
     break;
 
   case 8:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 227 "parser.y"
     {
 	    (mu_cfg_yyval.pnode) = mu_cfg_alloc_node (mu_cfg_node_statement, &(mu_cfg_yyvsp[(1) - (5)].ident).locus,
@@ -1590,7 +1622,8 @@ mu_cfg_yyreduce:
     break;
 
   case 9:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 233 "parser.y"
     {
 	    (mu_cfg_yyval.pnode) = mu_cfg_alloc_node (mu_cfg_node_statement, &(mu_cfg_yyvsp[(1) - (6)].ident).locus,
@@ -1600,7 +1633,8 @@ mu_cfg_yyreduce:
     break;
 
   case 10:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 241 "parser.y"
     {
 	    (mu_cfg_yyval.ident).name = (mu_cfg_yyvsp[(1) - (1)].string);
@@ -1609,7 +1643,8 @@ mu_cfg_yyreduce:
     break;
 
   case 11:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 248 "parser.y"
     {
 	    (mu_cfg_yyval.pvalue) = NULL;
@@ -1617,7 +1652,8 @@ mu_cfg_yyreduce:
     break;
 
   case 13:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 255 "parser.y"
     {
 	    size_t n = 0;
@@ -1655,7 +1691,8 @@ mu_cfg_yyreduce:
     break;
 
   case 14:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 291 "parser.y"
     {
 	    int rc = mu_list_create (&(mu_cfg_yyval.list));
@@ -1671,7 +1708,8 @@ mu_cfg_yyreduce:
     break;
 
   case 15:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 303 "parser.y"
     {
 	    mu_list_append ((mu_cfg_yyvsp[(1) - (2)].list), config_value_dup (&(mu_cfg_yyvsp[(2) - (2)].value)));
@@ -1679,7 +1717,8 @@ mu_cfg_yyreduce:
     break;
 
   case 16:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 309 "parser.y"
     {
 	      (mu_cfg_yyval.value).type = MU_CFG_STRING;
@@ -1688,7 +1727,8 @@ mu_cfg_yyreduce:
     break;
 
   case 17:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 314 "parser.y"
     {
 	      (mu_cfg_yyval.value).type = MU_CFG_LIST;
@@ -1697,7 +1737,8 @@ mu_cfg_yyreduce:
     break;
 
   case 18:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 319 "parser.y"
     {
 	      (mu_cfg_yyval.value).type = MU_CFG_STRING;
@@ -1706,7 +1747,8 @@ mu_cfg_yyreduce:
     break;
 
   case 22:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 331 "parser.y"
     {
 	    mu_iterator_t itr;
@@ -1727,7 +1769,8 @@ mu_cfg_yyreduce:
     break;
 
   case 23:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 350 "parser.y"
     {
 	    mu_list_create (&(mu_cfg_yyval.list));
@@ -1736,7 +1779,8 @@ mu_cfg_yyreduce:
     break;
 
   case 24:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 355 "parser.y"
     {
 	    mu_list_append ((mu_cfg_yyvsp[(1) - (2)].list), (mu_cfg_yyvsp[(2) - (2)].string));
@@ -1745,7 +1789,8 @@ mu_cfg_yyreduce:
     break;
 
   case 25:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 362 "parser.y"
     {
 	      (mu_cfg_yyval.list) = (mu_cfg_yyvsp[(2) - (3)].list);
@@ -1753,7 +1798,8 @@ mu_cfg_yyreduce:
     break;
 
   case 26:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 366 "parser.y"
     {
 	      (mu_cfg_yyval.list) = (mu_cfg_yyvsp[(2) - (4)].list);
@@ -1761,7 +1807,8 @@ mu_cfg_yyreduce:
     break;
 
   case 27:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 372 "parser.y"
     {
 	    mu_list_create (&(mu_cfg_yyval.list));
@@ -1770,7 +1817,8 @@ mu_cfg_yyreduce:
     break;
 
   case 28:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 377 "parser.y"
     {
 	    mu_list_append ((mu_cfg_yyvsp[(1) - (3)].list), config_value_dup (&(mu_cfg_yyvsp[(3) - (3)].value)));
@@ -1779,8 +1827,9 @@ mu_cfg_yyreduce:
     break;
 
 
-/* Line 1792 of yacc.c  */
-#line 1784 "parser.c"
+
+/* Line 1810 of yacc.c  */
+#line 1833 "parser.c"
       default: break;
     }
   /* User semantic actions sometimes alter mu_cfg_yychar, and that requires
@@ -1943,9 +1992,7 @@ mu_cfg_yyerrlab1:
       YY_STACK_PRINT (mu_cfg_yyss, mu_cfg_yyssp);
     }
 
-  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   *++mu_cfg_yyvsp = mu_cfg_yylval;
-  YY_IGNORE_MAYBE_UNINITIALIZED_END
 
 
   /* Shift the error token.  */
@@ -2011,7 +2058,8 @@ mu_cfg_yyreturn:
 }
 
 
-/* Line 2055 of yacc.c  */
+
+/* Line 2071 of yacc.c  */
 #line 388 "parser.y"
 
 
@@ -3090,7 +3138,7 @@ mu_cfg_find_node (mu_cfg_tree_t *tree, const char *path, mu_cfg_node_t **pval)
 	  *pval = (mu_cfg_node_t *) data.node;
 	  return 0;
 	}
-      else
+      else if (rc != 0)
 	mu_diag_funcall (MU_DIAG_ERR, "mu_cfg_preorder", NULL, rc);
     }
   return MU_ERR_NOENT;
@@ -3244,3 +3292,4 @@ mu_cfg_parse_config (mu_cfg_tree_t **ptree, struct mu_cfg_parse_hints *hints)
 	  
         
   
+
