@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007, 2009-2012, 2014-2016 Free Software
+   Copyright (C) 2005, 2007, 2009-2012, 2014-2017 Free Software
    Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -51,7 +51,7 @@
    is the message text.
 */
 static int
-build_mime (mu_sieve_machine_t mach, mu_list_t tags, mu_mime_t *pmime,
+build_mime (mu_sieve_machine_t mach, mu_mime_t *pmime,
 	    mu_message_t msg, const char *text)
 {
   mu_mime_t mime = NULL;
@@ -78,7 +78,7 @@ build_mime (mu_sieve_machine_t mach, mu_list_t tags, mu_mime_t *pmime,
       return 1;
     }
 
-  if (mu_sieve_tag_lookup (tags, "mime", NULL))
+  if (mu_sieve_get_tag (mach, "mime", SVT_VOID, NULL))
     {
       mu_stream_t fstr;
       rc = mu_filter_create (&fstr, input, "base64",
@@ -136,11 +136,6 @@ build_mime (mu_sieve_machine_t mach, mu_list_t tags, mu_mime_t *pmime,
 static int
 diag (mu_sieve_machine_t mach)
 {
-  if (mu_sieve_get_debug_level (mach) & MU_SIEVE_DEBUG_TRACE)
-    {
-      mu_sieve_debug (mach, "VACATION");
-    }
-
   mu_sieve_log_action (mach, "VACATION", NULL);
   return mu_sieve_is_dry_run (mach);
 }
@@ -166,8 +161,8 @@ _compare (void *item, void *data)
    of the originating mail. Return non-zero if so and store a pointer
    to the matching address in *MY_ADDRESS. */
 static int
-match_addresses (mu_header_t hdr, char *email, mu_sieve_value_t *addresses,
-		 char const **my_address)
+match_addresses (mu_sieve_machine_t mach, mu_header_t hdr, char *email,
+		 mu_sieve_value_t *addresses, char const **my_address)
 {
   int match = 0;
   const char *str;
@@ -181,7 +176,7 @@ match_addresses (mu_header_t hdr, char *email, mu_sieve_value_t *addresses,
 	  if (_compare (email, &ad))
 	    match = 1;
 	  else if (addresses)
-	    match += mu_sieve_vlist_do (addresses, _compare, &ad);
+	    match += mu_sieve_vlist_do (mach, addresses, _compare, &ad);
 	  mu_address_destroy (&ad.addr);
 	}
     }
@@ -193,7 +188,7 @@ match_addresses (mu_header_t hdr, char *email, mu_sieve_value_t *addresses,
 	  if (_compare (email, &ad))
 	    match = 1;
 	  else if (addresses)
-	    match += mu_sieve_vlist_do (addresses, _compare, &ad);
+	    match += mu_sieve_vlist_do (mach, addresses, _compare, &ad);
 	  mu_address_destroy (&ad.addr);
 	}
     }
@@ -232,7 +227,7 @@ regex_comparator (void *item, void *data)
 /* Decide whether EMAIL address should not be responded to.
  */
 static int
-noreply_address_p (mu_sieve_machine_t mach, mu_list_t tags, char *email)
+noreply_address_p (mu_sieve_machine_t mach, char *email)
 {
   int i, rc = 0;
   mu_sieve_value_t *arg;
@@ -254,8 +249,8 @@ noreply_address_p (mu_sieve_machine_t mach, mu_list_t tags, char *email)
   for (i = 0; rc == 0 && noreply_sender[i]; i++)
     rc = regex_comparator (noreply_sender[i], &rd);
 
-  if (!rc && mu_sieve_tag_lookup (tags, "noreply", &arg))
-    rc = mu_sieve_vlist_do (arg, regex_comparator, &rd);
+  if (!rc && (arg = mu_sieve_get_tag_untyped (mach, "noreply")) != NULL)
+    rc = mu_sieve_vlist_do (mach, arg, regex_comparator, &rd);
   
   return rc;
 }
@@ -342,28 +337,27 @@ test_and_update_prop (mu_property_t prop, const char *from,
    be answered, 0 if it should not, and throw exception if an error
    occurs. */
 static int
-check_db (mu_sieve_machine_t mach, mu_list_t tags, char *from)
+check_db (mu_sieve_machine_t mach, char *from)
 {
   mu_property_t prop;
   char *file;
-  mu_sieve_value_t *arg;
   unsigned int days;
   int rc;
   mu_stream_t str;
   mu_locker_t locker;
   const char *dbfile = "~/.vacation";
+  size_t n;
   
-  if (mu_sieve_tag_lookup (tags, "days", &arg))
+  if (mu_sieve_get_tag (mach, "days", SVT_NUMBER, &n))
     {
-      days = arg->v.number;
+      days = n;
       if (days > DAYS_MAX)
 	days = DAYS_MAX;
     }
   else
     days = DAYS_DEFAULT;
 
-  if (mu_sieve_tag_lookup (tags, "database", &arg))
-    dbfile = arg->v.string;
+  mu_sieve_get_tag (mach, "database", SVT_STRING, &dbfile);
   
   file = mu_tilde_expansion (dbfile, MU_HIERARCHY_DELIMITER, NULL);
   if (!file)
@@ -435,16 +429,12 @@ check_db (mu_sieve_machine_t mach, mu_list_t tags, char *from)
    "reply_prefix" tag.
  */
 static void
-re_subject (mu_sieve_machine_t mach, mu_list_t tags, char **psubject)
+re_subject (mu_sieve_machine_t mach, char **psubject)
 {
   char *subject;
-  mu_sieve_value_t *arg;
   char *prefix = "Re";
   
-  if (mu_sieve_tag_lookup (tags, "reply_prefix", &arg))
-    {
-      prefix = arg->v.string;
-    }
+  mu_sieve_get_tag (mach, "reply_prefix", SVT_STRING, &prefix);
 
   subject = malloc (strlen (*psubject) + strlen (prefix) + 3);
   if (!subject)
@@ -469,23 +459,22 @@ re_subject (mu_sieve_machine_t mach, mu_list_t tags, char **psubject)
    Otherwise, reply_prefix is prepended to it. */
 
 static void
-vacation_subject (mu_sieve_machine_t mach, mu_list_t tags,
+vacation_subject (mu_sieve_machine_t mach,
 		  mu_message_t msg, mu_header_t newhdr)
 {
-  mu_sieve_value_t *arg;
   char *value;
   char *subject;
   int subject_allocated = 0;
   mu_header_t hdr;
   
-  if (mu_sieve_tag_lookup (tags, "subject", &arg))
-    subject = arg->v.string;
+  if (mu_sieve_get_tag (mach, "subject", SVT_STRING, &subject))
+    /* nothing */;
   else if (mu_message_get_header (msg, &hdr) == 0
 	   && mu_header_aget_value_unfold (hdr, MU_HEADER_SUBJECT,
 					   &value) == 0)
     {
       char *p;
-      
+
       int rc = mu_rfc2047_decode (MU_SIEVE_CHARSET, value, &p);
 
       subject_allocated = 1;
@@ -499,11 +488,11 @@ vacation_subject (mu_sieve_machine_t mach, mu_list_t tags,
 	  subject = p;
 	}
 
-      if (mu_sieve_tag_lookup (tags, "reply_regex", &arg))
+      if (mu_sieve_get_tag (mach, "reply_regex", SVT_STRING, &p))
 	{
 	  char *err = NULL;
 	  
-	  rc = mu_unre_set_regex (arg->v.string, 0, &err);
+	  rc = mu_unre_set_regex (p, 0, &err);
 	  if (rc)
 	    {
 	      mu_sieve_error (mach,
@@ -515,7 +504,7 @@ vacation_subject (mu_sieve_machine_t mach, mu_list_t tags,
 	}
 	  
       if (mu_unre_subject (subject, NULL))
-	re_subject (mach, tags, &subject);
+	re_subject (mach, &subject);
       
       free (value);
     }
@@ -524,10 +513,10 @@ vacation_subject (mu_sieve_machine_t mach, mu_list_t tags,
     
   if (mu_rfc2047_encode (MU_SIEVE_CHARSET, "quoted-printable",
 			 subject, &value))
-    mu_header_set_value (newhdr, MU_HEADER_SUBJECT, subject, 0);
+    mu_header_set_value (newhdr, MU_HEADER_SUBJECT, subject, 1);
   else
     {
-      mu_header_set_value (newhdr, MU_HEADER_SUBJECT, value, 0);
+      mu_header_set_value (newhdr, MU_HEADER_SUBJECT, value, 1);
       free (value);
     }
 
@@ -612,7 +601,7 @@ add_header (void *item, void *data)
 
 /* Generate and send the reply message */
 static int
-vacation_reply (mu_sieve_machine_t mach, mu_list_t tags, mu_message_t msg,
+vacation_reply (mu_sieve_machine_t mach, mu_message_t msg,
 		char const *text, char const *to, char const *from)
 {
   mu_mime_t mime = NULL;
@@ -624,7 +613,7 @@ vacation_reply (mu_sieve_machine_t mach, mu_list_t tags, mu_message_t msg,
   int rc;
   mu_sieve_value_t *val;
 
-  if (mu_sieve_tag_lookup (tags, "file", NULL))
+  if (mu_sieve_get_tag (mach, "file", SVT_VOID, NULL))
     {
       mu_stream_t instr;
       
@@ -639,7 +628,7 @@ vacation_reply (mu_sieve_machine_t mach, mu_list_t tags, mu_message_t msg,
 	  return -1;
 	}
       
-      if (mu_sieve_tag_lookup (tags, "rfc2822", NULL))
+      if (mu_sieve_get_tag (mach, "rfc2822", SVT_VOID, NULL))
 	{
 	  rc = mu_stream_to_message (instr, &newmsg);
 	  mu_stream_unref (instr);
@@ -695,7 +684,7 @@ vacation_reply (mu_sieve_machine_t mach, mu_list_t tags, mu_message_t msg,
 	      return -1;
 	    }
 
-	  if (build_mime (mach, tags, &mime, msg, (char const *) trans[0]))
+	  if (build_mime (mach, &mime, msg, (char const *) trans[0]))
 	    {
 	      mu_stream_unref (text_stream);
 	      return -1;
@@ -707,7 +696,7 @@ vacation_reply (mu_sieve_machine_t mach, mu_list_t tags, mu_message_t msg,
     }
   else
     {
-      if (build_mime (mach, tags, &mime, msg, text))
+      if (build_mime (mach, &mime, msg, text))
 	return -1;
       mu_mime_get_message (mime, &newmsg);
       mu_message_unref (newmsg);
@@ -725,17 +714,18 @@ vacation_reply (mu_sieve_machine_t mach, mu_list_t tags, mu_message_t msg,
     }
   else
     {
-      mu_header_set_value (newhdr, MU_HEADER_TO, to, 0);
+      mu_header_set_value (newhdr, MU_HEADER_TO, to, 1);
 
-      if (mu_sieve_tag_lookup (tags, "header", &val))
+      val = mu_sieve_get_tag_untyped (mach, "header");
+      if (val)
 	{
 	  struct header_closure hc;
 	  hc.mach = mach;
 	  hc.hdr = newhdr;
-	  mu_sieve_vlist_do (val, add_header, &hc);
+	  mu_sieve_vlist_do (mach, val, add_header, &hc);
 	}
       
-      vacation_subject (mach, tags, msg, newhdr);
+      vacation_subject (mach, msg, newhdr);
       
       if (from)
         {
@@ -774,12 +764,11 @@ vacation_reply (mu_sieve_machine_t mach, mu_list_t tags, mu_message_t msg,
 }
 
 int
-sieve_action_vacation (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
+sieve_action_vacation (mu_sieve_machine_t mach)
 {
   int rc;
   char *text, *from = NULL;
   char const *return_address;
-  mu_sieve_value_t *val;
   mu_message_t msg;
   mu_header_t hdr;
   char *my_address;
@@ -787,22 +776,15 @@ sieve_action_vacation (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
   if (diag (mach))
     return 0;
   
-  val = mu_sieve_value_get (args, 0);
-  if (!val)
-    {
-      mu_sieve_error (mach, _("cannot get text!"));
-      mu_sieve_abort (mach);
-    }
-  else
-    text = val->v.string;
+  mu_sieve_get_arg (mach, 0, SVT_STRING, &text);
 
   msg = mu_sieve_get_message (mach);
   mu_message_get_header (msg, &hdr);
 
-  if (mu_sieve_tag_lookup (tags, "sender", &val))
+  if (mu_sieve_get_tag (mach, "sender", SVT_STRING, &from))
     {
       /* Debugging hook: :sender sets fake reply address */
-      from = strdup (val->v.string);
+      from = strdup (from);
       if (!from)
         {
           mu_sieve_error (mach, "%lu: %s",
@@ -822,32 +804,31 @@ sieve_action_vacation (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
 
   my_address = mu_get_user_email (NULL);
 
-  if (mu_sieve_tag_lookup (tags, "always_reply", NULL))
+  if (mu_sieve_get_tag (mach, "always_reply", SVT_VOID, NULL))
     return_address = my_address;
   else
     {
-      val = NULL;
-      mu_sieve_tag_lookup (tags, "aliases", &val);
-      if (match_addresses (hdr, my_address, val, &return_address) == 0)
+      mu_sieve_value_t *val = mu_sieve_get_tag_untyped (mach, "aliases");
+      if (match_addresses (mach, hdr, my_address, val, &return_address) == 0)
 	{
 	  free (my_address);
 	  return 0;
 	}
     }
 
-  if (noreply_address_p (mach, tags, from)
+  if (noreply_address_p (mach, from)
       || bulk_precedence_p (hdr)
-      || check_db (mach, tags, from))
+      || check_db (mach, from))
     {
       free (from);
       free (my_address);
       return 0;
     }
 
-  if (mu_sieve_tag_lookup (tags, "return_address", &val))
-    return_address = val->v.string;
+  mu_sieve_get_tag (mach, "return_address", SVT_STRING,
+		       &return_address);
 
-  rc = vacation_reply (mach, tags, msg, text, from, return_address);
+  rc = vacation_reply (mach, msg, text, from, return_address);
   free (from);
   free (my_address);
   if (rc == -1)
@@ -887,6 +868,7 @@ static mu_sieve_data_type vacation_args[] = {
 
 int SIEVE_EXPORT (vacation, init) (mu_sieve_machine_t mach)
 {
-  return mu_sieve_register_action (mach, "vacation", sieve_action_vacation,
-				vacation_args, vacation_tag_groups, 1);
+  mu_sieve_register_action (mach, "vacation", sieve_action_vacation,
+			    vacation_args, vacation_tag_groups, 1);
+  return 0;
 }
