@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 2.7.  */
+/* A Bison parser, made by GNU Bison 2.5.1.  */
 
 /* Bison interface for Yacc-like parsers in C
    
@@ -30,15 +30,6 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_YY_Y_TAB_H_INCLUDED
-# define YY_YY_Y_TAB_H_INCLUDED
-/* Enabling traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 1
-#endif
-#if YYDEBUG
-extern int mu_sieve_yydebug;
-#endif
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -57,7 +48,9 @@ extern int mu_sieve_yydebug;
      ELSE = 266,
      ANYOF = 267,
      ALLOF = 268,
-     NOT = 269
+     NOT = 269,
+     FALSE = 270,
+     TRUE = 271
    };
 #endif
 /* Tokens.  */
@@ -73,38 +66,36 @@ extern int mu_sieve_yydebug;
 #define ANYOF 267
 #define ALLOF 268
 #define NOT 269
+#define FALSE 270
+#define TRUE 271
+
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-/* Line 2058 of yacc.c  */
-#line 34 "sieve.y"
+
+/* Line 2072 of yacc.c  */
+#line 82 "sieve.y"
 
   char *string;
   size_t number;
-  sieve_instr_t instr;
-  mu_sieve_value_t *value;
-  mu_list_t list;
-  size_t pc;
-  struct {
-    size_t start;
-    size_t end;
-  } pclist;
-  struct {
+  size_t idx;
+  struct mu_sieve_slice slice;
+  struct
+  {
     char *ident;
-    mu_list_t args;
+    size_t first;
+    size_t count;
   } command;
-  struct {
-    size_t begin;
-    size_t cond;
-    size_t branch;
-  } branch;
+  struct mu_sieve_node_list node_list;
+  struct mu_sieve_node *node;
 
 
-/* Line 2058 of yacc.c  */
-#line 108 "sieve-gram.h"
+
+/* Line 2072 of yacc.c  */
+#line 99 "sieve-gram.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define mu_sieve_yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -113,18 +104,18 @@ typedef union YYSTYPE
 
 extern YYSTYPE mu_sieve_yylval;
 
-#ifdef YYPARSE_PARAM
-#if defined __STDC__ || defined __cplusplus
-int mu_sieve_yyparse (void *YYPARSE_PARAM);
-#else
-int mu_sieve_yyparse ();
+#if ! defined YYLTYPE && ! defined YYLTYPE_IS_DECLARED
+typedef struct YYLTYPE
+{
+  int first_line;
+  int first_column;
+  int last_line;
+  int last_column;
+} YYLTYPE;
+# define mu_sieve_yyltype YYLTYPE /* obsolescent; will be withdrawn */
+# define YYLTYPE_IS_DECLARED 1
+# define YYLTYPE_IS_TRIVIAL 1
 #endif
-#else /* ! YYPARSE_PARAM */
-#if defined __STDC__ || defined __cplusplus
-int mu_sieve_yyparse (void);
-#else
-int mu_sieve_yyparse ();
-#endif
-#endif /* ! YYPARSE_PARAM */
 
-#endif /* !YY_YY_Y_TAB_H_INCLUDED  */
+extern YYLTYPE mu_sieve_yylloc;
+
