@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-   2009, 2010 Free Software Foundation, Inc.
+   Copyright (C) 1999-2007, 2009-2012, 2014-2017 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,12 +13,10 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
-#include <mailutils/mutil.h>
+#include <mailutils/util.h>
 #include <mailutils/mime.h>
 #include <pwd.h>
 #ifdef HAVE_TERMIOS_H
@@ -32,28 +30,25 @@
 #else
 # include <sys/fcntl.h>
 #endif
-#include <mu_asprintf.h>
+#include <mailutils/io.h>
 
-/*
- * expands command into its command and arguments, then runs command
- * cmd is the command to parse and run
- * returns exit status of the command
- */
 int
-util_do_command (const char *c, ...)
+util_do_command (const char *fmt, ...)
 {
-  int argc = 0;
-  char **argv = NULL;
+  struct mu_wordsplit ws;
+  int argc;
+  char **argv;
   int status = 0;
   const struct mail_command_entry *entry = NULL;
   char *cmd = NULL;
+  size_t size = 0;
   va_list ap;
   
-  va_start (ap, c);
-  status = vasprintf (&cmd, c, ap);
+  va_start (ap, fmt);
+  status = mu_vasnprintf (&cmd, &size, fmt, ap);
   va_end (ap);
-  if (status < 0)
-    return 0;
+  if (status)
+    return status;
 
   if (cmd)
     {
@@ -72,24 +67,34 @@ util_do_command (const char *c, ...)
 	     according to the POSIX spec. Note, that this applies
 	     to interactive state only. */
 	  if (interactive)
-	    cmd = strdup ("next");
+	    cmd = mu_strdup ("next");
 	  else
 	    return 0;
 	}
 
-      if (mu_argcv_get (cmd, NULL, NULL, &argc, &argv) == 0 && argc > 0)
+      ws.ws_offs = 1; /* Keep one extra slot in case we need it
+			 for expansion (see below) */
+      if (mu_wordsplit (cmd, &ws, MU_WRDSF_DEFFLAGS|MU_WRDSF_DOOFFS))
+	{
+	  mu_error (_("%s failed: %s"), "mu_wordsplit",
+		    mu_wordsplit_strerror (&ws));
+	}
+      else
 	{
 	  char *p;
 
+	  argc = ws.ws_wordc;
+	  argv = ws.ws_wordv + 1;
+	  
 	  /* Special case: a number alone implies "print" */
 	  if (argc == 1
 	      && ((strtoul (argv[0], &p, 10) > 0 && *p == 0)
 		  || (argv[0][1] == 0 && strchr("^$", argv[0][0]))))
 	    {
-	      asprintf (&p, "print %s", argv[0]);
-	      mu_argcv_free (argc, argv);
-	      mu_argcv_get (p, NULL, NULL, &argc, &argv);
-	      free (p);
+	      /* Use the extra slot for "print" command */
+	      argc++;
+	      argv--;
+	      argv[0] = "print";
 	    }
 
 	  entry = mail_find_command (argv[0]);
@@ -115,12 +120,16 @@ util_do_command (const char *c, ...)
       
       if (strlen (p))
 	{
+	  /* Expand contracted form.  That's what we have kept an extra
+	     ws slot for. */
 	  argc++;
-	  argv = xrealloc (argv, (argc + 1)*sizeof argv[0]);
-	  if (argc > 2)
-	    memmove (argv + 2, argv + 1, (argc - 2)*sizeof argv[0]);
-	  argv[1] = xstrdup (p);
+	  argv--;
+	  argv[0] = argv[1];
+	  argv[1] = mu_strdup (p);
 	  *p = 0;
+	  /* Register the new entry in WS */
+	  ws.ws_wordc++;
+	  ws.ws_offs = 0;
 	}
       
       entry = mail_find_command (argv[0]);
@@ -135,13 +144,13 @@ util_do_command (const char *c, ...)
   else
     {
       if (argc)
-	util_error (_("Unknown command: %s"), argv[0]);
+	mu_error (_("Unknown command: %s"), argv[0]);
       else
-	util_error (_("Invalid command"));
+	mu_error (_("Invalid command"));
       status = 1;
     }
 
-  mu_argcv_free (argc, argv);
+  mu_wordsplit_free (&ws);
   return status;
 }
 
@@ -204,7 +213,7 @@ util_range_msg (size_t low, size_t high, int flags,
      if ((flags & MSG_NODELETED) && util_isdeleted (low))
        {
 	 if (!(flags & MSG_SILENT))
-	   util_error (_("%lu: Inappropriate message (has been deleted)"),
+	   mu_error (_("%lu: Inappropriate message (has been deleted)"),
 		       (unsigned long) low);
 	 continue;
        }
@@ -264,22 +273,20 @@ util_help (void *table, size_t nmemb, size_t size, const char *word)
   if (!word)
     {
       int i = 0;
-      FILE *out = stdout;
+      mu_stream_t out;
       char *p;
 
-      if (mailvar_get (NULL, "crt", mailvar_type_boolean, 0) == 0)
-	out = popen (getenv ("PAGER"), "w");
+      out = open_pager (util_screen_lines () + 1);
 
       for (p = table, i = 0; i < nmemb; i++, p += size)
 	{
 	  struct mail_command *cp = (struct mail_command *)p;
 	  if (cp->synopsis == NULL)
 	    continue;
-	  fprintf (out, "%s\n", cp->synopsis);
+	  mu_stream_printf (out, "%s\n", cp->synopsis);
 	}
       
-      if (out != stdout)
-	pclose (out);
+      mu_stream_unref (out);
 
       return 0;
     }
@@ -288,11 +295,11 @@ util_help (void *table, size_t nmemb, size_t size, const char *word)
       int status = 0;
       struct mail_command *cp = util_find_entry (table, nmemb, size, word);
       if (cp && cp->synopsis)
-	fprintf (stdout, "%s\n", cp->synopsis);
+	mu_printf ("%s\n", cp->synopsis);
       else
 	{
 	  status = 1;
-	  fprintf (stdout, _("Unknown command: %s\n"), word);
+	  mu_printf (_("Unknown command: %s\n"), word);
 	}
       return status;
     }
@@ -304,7 +311,7 @@ util_command_list (void *table, size_t nmemb, size_t size)
 {
   int i;
   char *p;
-  int cols = util_getcols ();
+  int cols = util_screen_columns ();
   int pos;
   
   for (p = table, i = 0; i < nmemb; i++, p += size)
@@ -325,12 +332,12 @@ util_command_list (void *table, size_t nmemb, size_t size)
       if (pos >= cols)
 	{
 	  pos = len + 1;
-	  fprintf (ofile, "\n%s ", cmd);
+	  mu_printf ("\n%s ", cmd);
 	}
       else
-        fprintf (ofile, "%s ", cmd);
+        mu_printf ("%s ", cmd);
     }
-  fprintf (ofile, "\n");
+  mu_printf ("\n");
   return 0;
 }
 
@@ -342,16 +349,9 @@ int
 util_getcols (void)
 {
   struct winsize ws;
-
   ws.ws_col = ws.ws_row = 0;
-  if ((ioctl(1, TIOCGWINSZ, (char *) &ws) < 0) || ws.ws_row == 0)
-    {
-      const char *columns = getenv ("COLUMNS");
-      if (columns)
-	ws.ws_col = strtol (columns, NULL, 10);
-    }
-
-  /* FIXME: Should we exit()/abort() if col <= 0 ?  */
+  if ((ioctl(1, TIOCGWINSZ, (char *) &ws) < 0) || ws.ws_col == 0)
+    return 80;  /* FIXME: Should we exit()/abort() if col <= 0 ?  */
   return ws.ws_col;
 }
 
@@ -363,17 +363,9 @@ int
 util_getlines (void)
 {
   struct winsize ws;
-
   ws.ws_col = ws.ws_row = 0;
-  if ((ioctl(1, TIOCGWINSZ, (char *) &ws) < 0) || ws.ws_row == 0)
-    {
-      const char *lines = getenv ("LINES");
-      if (lines)
-	ws.ws_row = strtol (lines, NULL, 10);
-    }
-
-  /* FIXME: Should we exit()/abort() if row <= 0 ?  */
-
+  if ((ioctl(1, TIOCGWINSZ, (char *) &ws) < 0) || ws.ws_row <= 2)
+    ws.ws_row = 24;  /* FIXME: Should we exit()/abort() if row <= 2 ?  */
   /* Reserve at least 2 line for the prompt.  */
   return ws.ws_row - 2;
 }
@@ -411,7 +403,7 @@ util_get_crt ()
 
   if (mailvar_get (&lines, "crt", mailvar_type_number, 0) == 0)
     return lines;
-  else if (mailvar_get (NULL, "crt", mailvar_type_boolean, 0) == 0)
+  else if (mailvar_is_true ("crt"))
     return util_getlines ();
   return 0;
 }
@@ -459,7 +451,7 @@ util_get_homedir ()
   if (!homedir)
     {
       /* Shouldn't happen, but one never knows */
-      util_error (_("Cannot get homedir"));
+      mu_error (_("Cannot get homedir"));
       exit (EXIT_FAILURE);
     }
   return homedir;
@@ -468,7 +460,7 @@ util_get_homedir ()
 char *
 util_fullpath (const char *inpath)
 {
-  return mu_tilde_expansion (inpath, "/", NULL);
+  return mu_tilde_expansion (inpath, MU_HIERARCHY_DELIMITER, NULL);
 }
 
 char *
@@ -489,15 +481,15 @@ util_folder_path (const char *name)
   if (folder[0] != '/' && folder[0] != '~')
     {
       char *home = mu_get_homedir ();
-      tmp  = xmalloc (strlen (home) + 1 +
-		      strlen (folder) + 1 +
-		      strlen (name) + 1);
+      tmp  = mu_alloc (strlen (home) + 1 +
+		       strlen (folder) + 1 +
+		       strlen (name) + 1);
       sprintf (tmp, "%s/%s/%s", home, folder, name);
     }
   else
     {
-      tmp  = xmalloc (strlen (folder) + 1 +
-		      strlen (name) + 1);
+      tmp  = mu_alloc (strlen (folder) + 1 +
+		       strlen (name) + 1);
       sprintf (tmp, "%s/%s", folder, name);
     }
   p = util_fullpath (tmp);
@@ -523,14 +515,14 @@ util_get_sender (int msgno, int strip)
       if (mu_envelope_sget_sender (env, &buffer)
 	  || mu_address_create (&addr, buffer))
 	{
-	  util_error (_("Cannot determine sender name (msg %d)"), msgno);
+	  mu_error (_("Cannot determine sender name (msg %d)"), msgno);
 	  return NULL;
 	}
     }
 
   if (mu_address_aget_email (addr, 1, &buf))
     {
-      util_error (_("Cannot determine sender name (msg %d)"), msgno);
+      mu_error (_("Cannot determine sender name (msg %d)"), msgno);
       mu_address_destroy (&addr);
       return NULL;
     }
@@ -558,8 +550,7 @@ util_slist_print (mu_list_t list, int nl)
   for (mu_iterator_first (itr); !mu_iterator_is_done (itr); mu_iterator_next (itr))
     {
       mu_iterator_current (itr, (void **)&name);
-      fprintf (ofile, "%s%c", name, nl ? '\n' : ' ');
-
+      mu_printf ("%s%c", name, nl ? '\n' : ' ');
     }
   mu_iterator_destroy (&itr);
 }
@@ -608,7 +599,7 @@ util_slist_add (mu_list_t *plist, char *value)
     }
   else
     list = *plist;
-  mu_list_append (list, xstrdup (value));
+  mu_list_append (list, mu_strdup (value));
 }
 
 void
@@ -649,7 +640,7 @@ void
 util_strcat (char **dest, const char *str)
 {
   if (!*dest)
-    *dest = strdup (str);
+    *dest = mu_strdup (str);
   else
     {
       int dlen = strlen (*dest) + 1;
@@ -687,7 +678,7 @@ util_outfolder_name (char *str)
       if (mailvar_get (&outfolder, "outfolder", mailvar_type_string, 0) == 0)
 	{
 	  char *ns = NULL;
-	  asprintf (&ns, "%s/%s", outfolder, str);
+	  mu_asprintf (&ns, "%s/%s", outfolder, str);
 	  str = util_fullpath (ns);
 	  free (ns);
 	}
@@ -695,7 +686,7 @@ util_outfolder_name (char *str)
 
     }
 
-  return strdup (str);
+  return mu_strdup (str);
 }
 
 /* Save an outgoing message. The SAVEFILE argument overrides the setting
@@ -714,22 +705,22 @@ util_save_outgoing (mu_message_t msg, char *savefile)
       rc = mu_mailbox_create_default (&outbox, filename);
       if (rc)
 	{
-	  util_error (_("Cannot create output mailbox `%s': %s"),
-		      filename, strerror (rc));
+	  mu_error (_("Cannot create output mailbox `%s': %s"),
+		      filename, mu_strerror (rc));
 	  free (filename);
 	  return;
 	}
 
       rc = mu_mailbox_open (outbox, MU_STREAM_WRITE | MU_STREAM_CREAT);
       if (rc)
-	util_error (_("Cannot open output mailbox `%s': %s"),
-		    filename, strerror (rc));
+	mu_error (_("Cannot open output mailbox `%s': %s"),
+		    filename, mu_strerror (rc));
       else
 	{
 	  rc = mu_mailbox_append_message (outbox, msg);
 	  if (rc)
-	    util_error (_("Cannot append message to `%s': %s"),
-			filename, strerror (rc));
+	    mu_error (_("Cannot append message to `%s': %s"),
+			filename, mu_strerror (rc));
 	}
 
       mu_mailbox_close (outbox);
@@ -739,19 +730,6 @@ util_save_outgoing (mu_message_t msg, char *savefile)
     }
 }
 
-void
-util_error (const char *format, ...)
-{
-  va_list ap;
-
-  va_start (ap, format);
-  
-  vfprintf (stderr, format, ap);
-  fprintf (stderr, "\n");
-
-  va_end(ap);
-}
-
 static int
 util_descend_subparts (mu_message_t mesg, msgset_t *msgset, mu_message_t *part)
 {
@@ -770,7 +748,7 @@ util_descend_subparts (mu_message_t mesg, msgset_t *msgset, mu_message_t *part)
 	{
 	  if (mu_message_unencapsulate (mesg, &submsg, NULL))
 	    {
-	      util_error (_("Cannot unencapsulate message/part"));
+	      mu_error (_("Cannot unencapsulate message/part"));
 	      return 1;
 	    }
 	  mesg = submsg;
@@ -779,14 +757,14 @@ util_descend_subparts (mu_message_t mesg, msgset_t *msgset, mu_message_t *part)
       mu_message_get_num_parts (mesg, &nparts);
       if (nparts < msgset->msg_part[i])
 	{
-	  util_error (_("No such (sub)part in the message: %lu"),
+	  mu_error (_("No such (sub)part in the message: %lu"),
 		      (unsigned long) msgset->msg_part[i]);
 	  return 1;
 	}
 
       if (mu_message_get_part (mesg, msgset->msg_part[i], &submsg))
 	{
-	  util_error (_("Cannot get (sub)part from the message: %lu"),
+	  mu_error (_("Cannot get (sub)part from the message: %lu"),
 		      (unsigned long) msgset->msg_part[i]);
 	  return 1;
 	}
@@ -824,7 +802,7 @@ util_get_content_type (mu_header_t hdr, char **value, char **args)
     {
       if (type)
 	free (type);
-      type = strdup ("text/plain"); /* Default.  */
+      type = mu_strdup ("text/plain"); /* Default.  */
     }
   else
     {
@@ -868,17 +846,13 @@ util_merge_addresses (char **addr_str, const char *value)
   rc = mu_address_union (&addr, new_addr);
   if (rc == 0)
     {
-      size_t n;
-
-      rc = mu_address_to_string (addr, NULL, 0, &n);
+      char *val;
+      
+      rc = mu_address_aget_printable (addr, &val);
       if (rc == 0)
 	{
 	  free (*addr_str);
-	  *addr_str = malloc (n + 1);
-	  if (!*addr_str)
-	    rc = ENOMEM;
-	  else
-	    mu_address_to_string (addr, *addr_str, n + 1, &n);
+	  *addr_str = val;
 	}
     }
 
@@ -911,90 +885,94 @@ util_header_expand (mu_header_t *phdr)
   mu_header_t hdr;
   int errcnt = 0, rc;
   
-  rc = mu_header_create (&hdr, "", 0, NULL);
+  rc = mu_header_create (&hdr, "", 0);
   if (rc)
     {
-      util_error (_("Cannot create temporary header: %s"), mu_strerror (rc));
+      mu_error (_("Cannot create temporary header: %s"), mu_strerror (rc));
       return 1;
     }
       
   mu_header_get_field_count (*phdr, &nfields);
   for (i = 1; i <= nfields; i++)
     {
-      char *name, *value;
+      const char *name, *value;
       
-      if (mu_header_aget_field_name (*phdr, i, &name))
+      if (mu_header_sget_field_name (*phdr, i, &name))
 	continue;
 
-      if (mu_header_aget_field_value (*phdr, i, &value))
-	{
-	  free (name);
-	  continue;
-	}
+      if (mu_header_sget_field_value (*phdr, i, &value))
+	continue;
       
       if (is_address_field (name))
 	{
-	  char *p, *s, *exp = NULL;
+	  const char *s;
 	  mu_address_t addr = NULL;
-
-	  if (mu_header_aget_value (hdr, name, &exp) == 0)
-	    {
-	      mu_address_create (&addr, exp);
-	      free (exp);
-	    }
+	  struct mu_wordsplit ws;
+	  size_t j;
 	  
-	  for (p = strtok_r (value, ",", &s); p; p = strtok_r (NULL, ",", &s))
+	  if (mu_header_sget_value (hdr, name, &s) == 0)
+	    mu_address_create (&addr, s);
+
+	  ws.ws_delim = ",";
+	  if (mu_wordsplit (value, &ws,
+			    MU_WRDSF_DELIM|MU_WRDSF_SQUEEZE_DELIMS|
+			    MU_WRDSF_WS|
+			    MU_WRDSF_NOVAR|MU_WRDSF_NOCMD))
 	    {
+	      errcnt++;
+	      mu_error (_("cannot split line `%s': %s"), value,
+			mu_wordsplit_strerror (&ws));
+	      break;
+	    }
+
+	  for (j = 0; j < ws.ws_wordc; j++)
+	    {
+	      const char *exp;
 	      mu_address_t new_addr;
+	      char *p = ws.ws_wordv[j];
 	      
-	      while (*p && mu_isspace (*p))
-		p++;
-	      /* If inplacealiases was set, the value was already expanded */
-	      if (mailvar_get (NULL, "inplacealiases", mailvar_type_boolean, 0))
+	      if (mailvar_is_true ("inplacealiases"))
+	        /* If inplacealiases was set, the value was already expanded */
+		exp = p;
+	      else
 		exp = alias_expand (p);
-	      rc = mu_address_create (&new_addr, exp ? exp : p);
+	      rc = mu_address_create (&new_addr, p);
 	      if (rc)
 		{
 		  errcnt++;
 		  if (exp)
-		    util_error (_("Cannot parse address `%s' (while expanding `%s'): %s"),
+		    mu_error (_("Cannot parse address `%s' (while expanding `%s'): %s"),
 				exp, p, mu_strerror (rc));
 		  else
-		    util_error (_("Cannot parse address `%s': %s"),
+		    mu_error (_("Cannot parse address `%s': %s"),
 				p, mu_strerror (rc));
 		}
 	      
-	      free (exp);
 	      mu_address_union (&addr, new_addr);
 	      mu_address_destroy (&new_addr);
 	    }
 	  
 	  if (addr)
 	    {
-	      size_t n = 0;
+	      const char *newvalue;
 	      
-	      free (value);
-	      mu_address_to_string (addr, NULL, 0, &n);
-	      value = xmalloc (n + 1);
-	      mu_address_to_string (addr, value, n + 1, NULL);
+	      rc = mu_address_sget_printable (addr, &newvalue);
+	      if (rc == 0)
+		mu_header_set_value (hdr, name, newvalue, 1);
 	      mu_address_destroy (&addr);
-	      mu_header_set_value (hdr, name, value, 1);
 	    }
 	}
       else
-	mu_header_set_value (hdr, name, value, 0);
-      
-      free (value);
-      free (name);
+	mu_header_append (hdr, name, value);
     }
 
   if (errcnt == 0)
     {
-      mu_header_destroy (phdr, NULL);
+      mu_header_destroy (phdr);
       *phdr = hdr;
     }
   else
-    mu_header_destroy (&hdr, NULL);
+    mu_header_destroy (&hdr);
 
   return errcnt;
 }
@@ -1013,7 +991,7 @@ util_get_message (mu_mailbox_t mbox, size_t msgno, mu_message_t *msg)
   status = mu_mailbox_get_message (mbox, msgno, msg);
   if (status)
     {
-      util_error (_("Cannot get message %lu: %s"),
+      mu_error (_("Cannot get message %lu: %s"),
 		  (unsigned long) msgno, mu_strerror (status));
       return status;
     }
@@ -1024,24 +1002,25 @@ util_get_message (mu_mailbox_t mbox, size_t msgno, mu_message_t *msg)
 int
 util_error_range (size_t msgno)
 {
-  util_error (_("%lu: invalid message number"), (unsigned long) msgno);
+  mu_error (_("%lu: invalid message number"), (unsigned long) msgno);
   return 1;
 }
 
 void
 util_noapp ()
 {
-  util_error (_("No applicable messages"));
+  mu_error (_("No applicable messages"));
 }
 
 void
 util_cache_command (mu_list_t *list, const char *fmt, ...)
 {
-  char *cmd;
+  char *cmd = NULL;
+  size_t size = 0;
   va_list ap;
 
   va_start (ap, fmt);
-  vasprintf (&cmd, fmt, ap);
+  mu_vasnprintf (&cmd, &size, fmt, ap);
   va_end (ap);
 
   if (!*list)
@@ -1061,55 +1040,57 @@ _run_and_free (void *item, void *data)
 void
 util_run_cached_commands (mu_list_t *list)
 {
-  mu_list_do (*list, _run_and_free, NULL);
+  mu_list_foreach (*list, _run_and_free, NULL);
   mu_list_destroy (list);
 }
 
+char *
+util_get_charset (void)
+{
+  char *charset;
+
+  if (mailvar_get (&charset, "charset", mailvar_type_string, 0))
+    return NULL;
+
+  if (mu_c_strcasecmp (charset, "auto") == 0)
+    {
+      struct mu_lc_all lc_all = { .flags = 0 };
+      char *tmp = getenv ("LC_ALL");
+      if (!tmp)
+	tmp = getenv ("LANG");
+      
+      if (tmp && mu_parse_lc_all (tmp, &lc_all, MU_LC_CSET) == 0)
+	{
+	  charset = mu_strdup (lc_all.charset);
+	  mu_lc_all_free (&lc_all);
+	}
+      else
+	charset = NULL;
+    }
+  else
+    charset = mu_strdup (charset);
+
+  return charset;
+}
+
 void
 util_rfc2047_decode (char **value)
 {
-  char locale[32];
-  const char *charset = NULL;
-  char *tmp;
+  char *charset, *tmp;
   int rc;
 
-  if (!*value || mailvar_get (&charset, "charset", mailvar_type_string, 0))
+  if (!*value)
     return;
-
-  if (mu_c_strcasecmp (charset, "auto") == 0)
-    {
-      memset (locale, 0, sizeof (locale));
-
-      /* Try to deduce the charset from LC_ALL or LANG variables */
-
-      tmp = getenv ("LC_ALL");
-      if (!tmp)
-	tmp = getenv ("LANG");
-
-      if (tmp)
-	{
-	  char *sp;
-	  char *lang;
-	  char *terr;
-
-	  strncpy (locale, tmp, sizeof (locale) - 1);
-
-	  lang = strtok_r (locale, "_", &sp);
-	  terr = strtok_r (NULL, ".", &sp);
-	  charset = strtok_r (NULL, "@", &sp);
-
-	  if (!charset)
-	    charset = mu_charset_lookup (lang, terr);
-	}
-    }
-
+  charset = util_get_charset ();
   if (!charset)
     return;
-  
+
   rc = mu_rfc2047_decode (charset, *value, &tmp);
+  free (charset);
+  
   if (rc)
     {
-      if (mailvar_get (NULL, "verbose", mailvar_type_boolean, 0) == 0)
+      if (mailvar_is_true ("verbose"))
 	mu_error (_("Cannot decode line `%s': %s"), *value, mu_strerror (rc));
     }
   else
@@ -1134,3 +1115,29 @@ util_url_to_string (mu_url_t url)
     }
   return mu_url_to_string (url);
 }
+
+mu_stream_t
+open_pager (size_t lines)
+{
+  const char *pager;
+  unsigned pagelines = util_get_crt ();
+  mu_stream_t str;
+  
+  if (pagelines && lines > pagelines && (pager = getenv ("PAGER")))
+    {
+      int rc = mu_command_stream_create (&str, pager, MU_STREAM_WRITE);
+      if (rc)
+	{
+	  mu_diag_funcall (MU_DIAG_ERROR, "mu_prog_stream_create",
+			   pager, rc);
+	  str = mu_strout;
+	  mu_stream_ref (str);
+	}
+    }
+  else
+    {
+      str = mu_strout;
+      mu_stream_ref (str);
+    }
+  return str;
+}
